@@ -5,7 +5,7 @@ use std::{
 };
 
 use async_trait::async_trait;
-use futures::{future::join_all};
+use futures::future::join_all;
 
 /// Event bus
 /// # Using example
@@ -26,24 +26,27 @@ use futures::{future::join_all};
 ///     // Create event bus
 ///     let mut event_bus = EventBus::new().await;
 ///     // Subscribe on our topic
-///     event_bus.subscribe::<Topic>(Box::new(SimpleSubscriber {})).await;
+///     event_bus.subscribe::<Topic>(&SimpleSubscriber {}).await;
 ///     // Publish event
 ///     event_bus.publish::<Topic>(&"Hello".to_string()).await;
 /// }
 /// ```
-pub struct EventBus(HashMap<TypeId, Topic>);
+pub struct EventBus<'a>(HashMap<TypeId, Topic<'a>>);
 
-impl EventBus {
+impl EventBus<'_> {
     /// Creates new event bus.
-    pub async fn new() -> EventBus {
+    pub async fn new() -> EventBus<'static> {
         EventBus { 0: HashMap::new() }
     }
-    async fn register_topic<T: 'static>(&mut self, topic: Topic) {
+    async fn register_topic<T: 'static>(&mut self, topic: Topic<'static>) {
         self.0.insert(TypeId::of::<T>(), topic);
     }
 
     /// Subscribes on topic.
-    pub async fn subscribe<T: 'static>(&mut self, subscriber: Box<dyn Subscriber + Send>) {
+    pub async fn subscribe<T: 'static>(
+        &mut self,
+        subscriber: &'static (dyn Subscriber + Send + Sync),
+    ) {
         match self.0.get_mut(&TypeId::of::<T>()) {
             Some(topic) => topic.0.push(subscriber),
             None => {
@@ -70,15 +73,17 @@ impl EventBus {
     }
 }
 
+unsafe impl Send for EventBus<'_> {}
+
 /// Topic in event bus
-struct Topic(Vec<Box<dyn Subscriber + Send>>);
-impl Topic {
+struct Topic<'a>(Vec<&'a (dyn Subscriber + Send)>);
+impl Topic<'_> {
     /// Publishes event.
     pub async fn publish(&mut self, message: &(dyn Any + Send + Sync)) {
         let mut futures = vec![];
         for i in 0..self.0.len() {
             if let Some(subscriber) = self.0.get(i) {
-                futures.push(subscriber.as_ref().handle(message));
+                futures.push(subscriber.handle(message));
             }
         }
 
@@ -86,6 +91,8 @@ impl Topic {
     }
 }
 
+unsafe impl Send for Topic<'_> {}
+
 /// Subscriber trait
 /// # Example
 ///
@@ -115,7 +122,9 @@ pub trait Subscriber {
 #[cfg(test)]
 mod tests {
 
-    use crate::{EventBus};
+    use futures::executor::block_on;
+
+    use crate::EventBus;
 
     mod subscribers {
         use std::any::Any;
@@ -149,16 +158,14 @@ mod tests {
 
     #[test]
     fn test_event_bus() {
-        futures::executor::ThreadPool::new()
-            .expect("Failed to create thread pool")
-            .spawn_ok(async {
-                let mut event_bus = EventBus::new().await;
-                event_bus
-                    .subscribe::<subscribers::Topic>(Box::new(subscribers::SimpleSubscriber {}))
-                    .await;
-                event_bus
-                    .publish::<subscribers::Topic>(&"Hello".to_string())
-                    .await;
-            });
+        block_on(async {
+            let mut event_bus = EventBus::new().await;
+            event_bus
+                .subscribe::<subscribers::Topic>(&subscribers::SimpleSubscriber {})
+                .await;
+            event_bus
+                .publish::<subscribers::Topic>(&"Hello".to_string())
+                .await;
+        });
     }
 }
